@@ -1,21 +1,17 @@
-# coding: utf-8
-from __future__ import unicode_literals
-
-import io
 import logging
 import os
 
 config_text = 'site_name: My Docs\n'
 index_text = """# Welcome to MkDocs
 
-For full documentation visit [mkdocs.org](https://mkdocs.org).
+For full documentation visit [mkdocs.org](https://www.mkdocs.org).
 
 ## Commands
 
 * `mkdocs new [dir-name]` - Create a new project.
 * `mkdocs serve` - Start the live-reloading docs server.
 * `mkdocs build` - Build the documentation site.
-* `mkdocs help` - Print this help message.
+* `mkdocs -h` - Print help message and exit.
 
 ## Project layout
 
@@ -43,7 +39,8 @@ def new(output_dir):
         os.mkdir(output_dir)
 
     log.info('Writing config file: %s', config_path)
-    io.open(config_path, 'w', encoding='utf-8').write(config_text)
+    with open(config_path, 'w', encoding='utf-8') as f:
+        f.write(config_text)
 
     if os.path.exists(index_path):
         return
@@ -51,4 +48,5 @@ def new(output_dir):
     log.info('Writing initial docs: %s', index_path)
     if not os.path.exists(docs_dir):
         os.mkdir(docs_dir)
-    io.open(index_path, 'w', encoding='utf-8').write(index_text)
+    with open(index_path, 'w', encoding='utf-8') as f:
+        f.write(index_text)
