@@ -1,19 +1,14 @@
 #!/usr/bin/env python
-# coding: utf-8
 
-from __future__ import unicode_literals
 import unittest
-import mock
+from unittest import mock
 import logging
-import sys
 import io
 
 from click.testing import CliRunner
 
 from mkdocs import __main__ as cli
 
-PY3 = sys.version_info[0] == 3
-
 
 class CLITests(unittest.TestCase):
 
@@ -28,12 +23,12 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='livereload',
+            config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
-            livereload='livereload'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -46,10 +41,7 @@ class CLITests(unittest.TestCase):
         self.assertEqual(mock_serve.call_count, 1)
         args, kwargs = mock_serve.call_args
         self.assertTrue('config_file' in kwargs)
-        if PY3:
-            self.assertIsInstance(kwargs['config_file'], io.BufferedReader)
-        else:
-            self.assertTrue(isinstance(kwargs['config_file'], file))
+        self.assertIsInstance(kwargs['config_file'], io.BufferedReader)
         self.assertEqual(kwargs['config_file'].name, 'mkdocs.yml')
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -60,12 +52,12 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr='0.0.0.0:80',
+            livereload='livereload',
+            config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
-            livereload='livereload'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -76,12 +68,12 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='livereload',
+            config_file=None,
             strict=True,
             theme=None,
-            theme_dir=None,
-            livereload='livereload'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -92,28 +84,44 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='livereload',
+            config_file=None,
             strict=None,
             theme='readthedocs',
-            theme_dir=None,
-            livereload='livereload'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
-    def test_serve_theme_dir(self, mock_serve):
+    def test_serve_use_directory_urls(self, mock_serve):
 
         result = self.runner.invoke(
-            cli.cli, ["serve", '--theme-dir', 'custom'], catch_exceptions=False)
+            cli.cli, ["serve", '--use-directory-urls'], catch_exceptions=False)
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='livereload',
+            config_file=None,
             strict=None,
             theme=None,
-            theme_dir='custom',
-            livereload='livereload'
+            use_directory_urls=True
+        )
+
+    @mock.patch('mkdocs.commands.serve.serve', autospec=True)
+    def test_serve_no_directory_urls(self, mock_serve):
+
+        result = self.runner.invoke(
+            cli.cli, ["serve", '--no-directory-urls'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        mock_serve.assert_called_once_with(
+            dev_addr=None,
+            livereload='livereload',
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=False
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -124,12 +132,12 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='livereload',
+            config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
-            livereload='livereload'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -140,12 +148,12 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='no-livereload',
+            config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
-            livereload='no-livereload'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.commands.serve.serve', autospec=True)
@@ -156,12 +164,12 @@ class CLITests(unittest.TestCase):
 
         self.assertEqual(result.exit_code, 0)
         mock_serve.assert_called_once_with(
-            config_file=None,
             dev_addr=None,
+            livereload='dirty',
+            config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
-            livereload='dirty'
+            use_directory_urls=None
         )
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -180,7 +188,7 @@ class CLITests(unittest.TestCase):
             config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
+            use_directory_urls=None,
             site_dir=None
         )
         logger = logging.getLogger('mkdocs')
@@ -224,10 +232,7 @@ class CLITests(unittest.TestCase):
         self.assertEqual(mock_load_config.call_count, 1)
         args, kwargs = mock_load_config.call_args
         self.assertTrue('config_file' in kwargs)
-        if PY3:
-            self.assertIsInstance(kwargs['config_file'], io.BufferedReader)
-        else:
-            self.assertTrue(isinstance(kwargs['config_file'], file))
+        self.assertIsInstance(kwargs['config_file'], io.BufferedReader)
         self.assertEqual(kwargs['config_file'].name, 'mkdocs.yml')
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -243,7 +248,7 @@ class CLITests(unittest.TestCase):
             config_file=None,
             strict=True,
             theme=None,
-            theme_dir=None,
+            use_directory_urls=None,
             site_dir=None
         )
 
@@ -260,16 +265,16 @@ class CLITests(unittest.TestCase):
             config_file=None,
             strict=None,
             theme='readthedocs',
-            theme_dir=None,
+            use_directory_urls=None,
             site_dir=None
         )
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
     @mock.patch('mkdocs.commands.build.build', autospec=True)
-    def test_build_theme_dir(self, mock_build, mock_load_config):
+    def test_build_use_directory_urls(self, mock_build, mock_load_config):
 
         result = self.runner.invoke(
-            cli.cli, ['build', '--theme-dir', 'custom'], catch_exceptions=False)
+            cli.cli, ['build', '--use-directory-urls'], catch_exceptions=False)
 
         self.assertEqual(result.exit_code, 0)
         self.assertEqual(mock_build.call_count, 1)
@@ -277,7 +282,24 @@ class CLITests(unittest.TestCase):
             config_file=None,
             strict=None,
             theme=None,
-            theme_dir='custom',
+            use_directory_urls=True,
+            site_dir=None
+        )
+
+    @mock.patch('mkdocs.config.load_config', autospec=True)
+    @mock.patch('mkdocs.commands.build.build', autospec=True)
+    def test_build_no_directory_urls(self, mock_build, mock_load_config):
+
+        result = self.runner.invoke(
+            cli.cli, ['build', '--no-directory-urls'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        self.assertEqual(mock_build.call_count, 1)
+        mock_load_config.assert_called_once_with(
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=False,
             site_dir=None
         )
 
@@ -294,8 +316,8 @@ class CLITests(unittest.TestCase):
             config_file=None,
             strict=None,
             theme=None,
-            theme_dir=None,
-            site_dir='custom'
+            use_directory_urls=None,
+            site_dir='custom',
         )
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -353,9 +375,13 @@ class CLITests(unittest.TestCase):
         self.assertTrue('dirty' in b_kwargs)
         self.assertFalse(b_kwargs['dirty'])
         mock_load_config.assert_called_once_with(
-            config_file=None,
             remote_branch=None,
-            remote_name=None
+            remote_name=None,
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=None,
+            site_dir=None
         )
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -402,10 +428,7 @@ class CLITests(unittest.TestCase):
         self.assertEqual(mock_load_config.call_count, 1)
         args, kwargs = mock_load_config.call_args
         self.assertTrue('config_file' in kwargs)
-        if PY3:
-            self.assertIsInstance(kwargs['config_file'], io.BufferedReader)
-        else:
-            self.assertTrue(isinstance(kwargs['config_file'], file))
+        self.assertIsInstance(kwargs['config_file'], io.BufferedReader)
         self.assertEqual(kwargs['config_file'].name, 'mkdocs.yml')
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -436,9 +459,13 @@ class CLITests(unittest.TestCase):
         self.assertEqual(mock_gh_deploy.call_count, 1)
         self.assertEqual(mock_build.call_count, 1)
         mock_load_config.assert_called_once_with(
-            config_file=None,
             remote_branch='foo',
-            remote_name=None
+            remote_name=None,
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=None,
+            site_dir=None
         )
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -453,9 +480,13 @@ class CLITests(unittest.TestCase):
         self.assertEqual(mock_gh_deploy.call_count, 1)
         self.assertEqual(mock_build.call_count, 1)
         mock_load_config.assert_called_once_with(
-            config_file=None,
             remote_branch=None,
-            remote_name='foo'
+            remote_name='foo',
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=None,
+            site_dir=None
         )
 
     @mock.patch('mkdocs.config.load_config', autospec=True)
@@ -477,7 +508,7 @@ class CLITests(unittest.TestCase):
     @mock.patch('mkdocs.config.load_config', autospec=True)
     @mock.patch('mkdocs.commands.build.build', autospec=True)
     @mock.patch('mkdocs.commands.gh_deploy.gh_deploy', autospec=True)
-    def test_gh_deploy_ognore_version(self, mock_gh_deploy, mock_build, mock_load_config):
+    def test_gh_deploy_ignore_version(self, mock_gh_deploy, mock_build, mock_load_config):
 
         result = self.runner.invoke(
             cli.cli, ['gh-deploy', '--ignore-version'], catch_exceptions=False)
@@ -489,3 +520,108 @@ class CLITests(unittest.TestCase):
         self.assertEqual(g_kwargs['ignore_version'], True)
         self.assertEqual(mock_build.call_count, 1)
         self.assertEqual(mock_load_config.call_count, 1)
+
+    @mock.patch('mkdocs.config.load_config', autospec=True)
+    @mock.patch('mkdocs.commands.build.build', autospec=True)
+    @mock.patch('mkdocs.commands.gh_deploy.gh_deploy', autospec=True)
+    def test_gh_deploy_strict(self, mock_gh_deploy, mock_build, mock_load_config):
+
+        result = self.runner.invoke(
+            cli.cli, ['gh-deploy', '--strict'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        self.assertEqual(mock_gh_deploy.call_count, 1)
+        self.assertEqual(mock_build.call_count, 1)
+        mock_load_config.assert_called_once_with(
+            remote_branch=None,
+            remote_name=None,
+            config_file=None,
+            strict=True,
+            theme=None,
+            use_directory_urls=None,
+            site_dir=None
+        )
+
+    @mock.patch('mkdocs.config.load_config', autospec=True)
+    @mock.patch('mkdocs.commands.build.build', autospec=True)
+    @mock.patch('mkdocs.commands.gh_deploy.gh_deploy', autospec=True)
+    def test_gh_deploy_theme(self, mock_gh_deploy, mock_build, mock_load_config):
+
+        result = self.runner.invoke(
+            cli.cli, ['gh-deploy', '--theme', 'readthedocs'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        self.assertEqual(mock_gh_deploy.call_count, 1)
+        self.assertEqual(mock_build.call_count, 1)
+        mock_load_config.assert_called_once_with(
+            remote_branch=None,
+            remote_name=None,
+            config_file=None,
+            strict=None,
+            theme='readthedocs',
+            use_directory_urls=None,
+            site_dir=None
+        )
+
+    @mock.patch('mkdocs.config.load_config', autospec=True)
+    @mock.patch('mkdocs.commands.build.build', autospec=True)
+    @mock.patch('mkdocs.commands.gh_deploy.gh_deploy', autospec=True)
+    def test_gh_deploy_use_directory_urls(self, mock_gh_deploy, mock_build, mock_load_config):
+
+        result = self.runner.invoke(
+            cli.cli, ['gh-deploy', '--use-directory-urls'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        self.assertEqual(mock_gh_deploy.call_count, 1)
+        self.assertEqual(mock_build.call_count, 1)
+        mock_load_config.assert_called_once_with(
+            remote_branch=None,
+            remote_name=None,
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=True,
+            site_dir=None
+        )
+
+    @mock.patch('mkdocs.config.load_config', autospec=True)
+    @mock.patch('mkdocs.commands.build.build', autospec=True)
+    @mock.patch('mkdocs.commands.gh_deploy.gh_deploy', autospec=True)
+    def test_gh_deploy_no_directory_urls(self, mock_gh_deploy, mock_build, mock_load_config):
+
+        result = self.runner.invoke(
+            cli.cli, ['gh-deploy', '--no-directory-urls'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        self.assertEqual(mock_gh_deploy.call_count, 1)
+        self.assertEqual(mock_build.call_count, 1)
+        mock_load_config.assert_called_once_with(
+            remote_branch=None,
+            remote_name=None,
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=False,
+            site_dir=None
+        )
+
+    @mock.patch('mkdocs.config.load_config', autospec=True)
+    @mock.patch('mkdocs.commands.build.build', autospec=True)
+    @mock.patch('mkdocs.commands.gh_deploy.gh_deploy', autospec=True)
+    def test_gh_deploy_site_dir(self, mock_gh_deploy, mock_build, mock_load_config):
+
+        result = self.runner.invoke(
+            cli.cli, ['gh-deploy', '--site-dir', 'custom'], catch_exceptions=False)
+
+        self.assertEqual(result.exit_code, 0)
+        self.assertEqual(mock_gh_deploy.call_count, 1)
+        self.assertEqual(mock_build.call_count, 1)
+        mock_load_config.assert_called_once_with(
+            remote_branch=None,
+            remote_name=None,
+            config_file=None,
+            strict=None,
+            theme=None,
+            use_directory_urls=None,
+            site_dir='custom'
+        )
